@@ -1,30 +1,55 @@
 from datetime import datetime, timedelta
 from os import PathLike
 from pathlib import Path
+from typing import Dict, List
 
 from .configuration import (
     ConfigurationFile,
     ensure_directory,
-    MeshFile,
+    FileForcingsFile,
     ModelConfigurationFile,
     NEMSConfigurationFile,
     RunSequence,
 )
-from .model.base import ConnectionEntry, MediationEntry, ModelEntry, ModelType, RemapMethod
+from .model.base import ConnectionEntry, EntryType, GridRemapMethod, MediationEntry, ModelEntry
 from .utilities import parse_datetime
 
 
 class ModelingSystem:
     """
-    NEMS interface with configuration file output
+    main user interface class of NEMSpy, providing configuration and output to files
+
+    >>> from datetime import datetime, timedelta
+    >>> from nemspy import ModelingSystem
+    >>> from nemspy.model import ADCIRCEntry, AtmosphericForcingEntry, WaveWatch3ForcingEntry
+    >>> # instantiate model system with model entries
+    >>> nems = ModelingSystem(
+    >>>     start_time=datetime(2020, 6, 1),
+    >>>     end_time=start_time + timedelta(days=1),
+    >>>     interval=timedelta(hours=1),
+    >>>     ocn=ADCIRCEntry(processors=11),
+    >>>     atm=AtmosphericForcingEntry(filename='wind_atm_fin_ch_time_vec.nc'),
+    >>>     wav=WaveWatch3ForcingEntry(filename='ww3.Constant.20151214_sxy_ike_date.nc'),
+    >>> )
+    >>> # form connections between models
+    >>> nems.connect('ATM', 'OCN')
+    >>> nems.connect('WAV', 'OCN')
+    >>> # define execution order
+    >>> nems.sequence = [
+    >>>     'ATM -> OCN',
+    >>>     'WAV -> OCN',
+    >>>     'ATM',
+    >>>     'WAV',
+    >>>     'OCN',
+    >>> ]
+    >>> # write configuration files to the given directory
+    >>> nems.write('nems_configuration')
     """
 
     def __init__(
         self, start_time: datetime, end_time: datetime, interval: timedelta, **models,
     ):
         """
-        create a NEMS interface from the given interval and models
-
         :param start_time: start time within the modeled system
         :param end_time: end time within the modeled system
         :param interval: time interval of top-level run sequence
@@ -39,7 +64,7 @@ class ModelingSystem:
         self.__start_time = start_time
         self.end_time = end_time
 
-        model_types = [model_type.value.lower() for model_type in ModelType]
+        model_types = [model_type.value.lower() for model_type in EntryType]
 
         parsed_models = {}
         attributes = {}
@@ -47,7 +72,7 @@ class ModelingSystem:
             if key.lower() in model_types:
                 key = key.lower()
                 if isinstance(value, ModelEntry):
-                    if value.model_type.value.lower() == key:
+                    if value.entry_type.value.lower() == key:
                         parsed_models[key.upper()] = value
                     else:
                         raise TypeError(f'"{value.name}" is not {key}')
@@ -61,7 +86,10 @@ class ModelingSystem:
 
     @property
     def start_time(self) -> datetime:
-        """ end time within modeled system """
+        """
+        end time within modeled system
+        """
+
         return self.__start_time
 
     @start_time.setter
@@ -74,7 +102,10 @@ class ModelingSystem:
 
     @property
     def end_time(self) -> datetime:
-        """ end time within modeled system """
+        """
+        end time within modeled system
+        """
+
         return self.__end_time
 
     @end_time.setter
@@ -87,12 +118,18 @@ class ModelingSystem:
 
     @property
     def duration(self) -> timedelta:
-        """ duration of run within modeled system """
+        """
+        duration of run within modeled system
+        """
+
         return self.end_time - self.start_time
 
     @property
     def interval(self) -> timedelta:
-        """ run sequence interval within the modeled system """
+        """
+        run sequence interval within the modeled system
+        """
+
         return self.__sequence.interval
 
     @interval.setter
@@ -104,26 +141,35 @@ class ModelingSystem:
         return self.__sequence.attributes
 
     @attributes.setter
-    def attributes(self, attributes: {str: str}):
+    def attributes(self, attributes: Dict[str, str]):
         self.__sequence.attributes = attributes
 
     @property
-    def models(self) -> [ModelEntry]:
-        """ models in execution order """
+    def models(self) -> List[ModelEntry]:
+        """
+        models in execution order
+        """
+
         return self.__sequence.models
 
     @property
     def processors(self) -> int:
-        """ number of PETs / processors / tasks """
+        """
+        number of PETs / processors / tasks
+        """
+
         return self.__sequence.processors
 
     @property
-    def sequence(self) -> [str]:
-        """ model execution order """
+    def sequence(self) -> List[str]:
+        """
+        model execution order
+        """
+
         return [entry.sequence_entry for entry in self.__sequence.sequence]
 
     @sequence.setter
-    def sequence(self, sequence: [str]):
+    def sequence(self, sequence: List[str]):
         sequence_entries = []
         entries = {entry.sequence_entry: entry for entry in self.__sequence.sequence}
         for entry in sequence:
@@ -133,15 +179,15 @@ class ModelingSystem:
                 models = [model.strip() for model in entry.split('->')]
                 if len(models) == 2:
                     connection = ConnectionEntry.from_string(entry)
-                    source = connection.source.model_type.value
-                    destination = connection.target.model_type.value
+                    source = connection.source.entry_type.value
+                    destination = connection.target.entry_type.value
                     del connection
 
                     for connection in self.__sequence.connections:
                         if isinstance(connection, ConnectionEntry):
                             if (
-                                connection.source.model_type.value == source
-                                and connection.target.model_type.value == destination
+                                connection.source.entry_type.value == source
+                                and connection.target.entry_type.value == destination
                             ):
                                 sequence_entries.append(connection)
                                 break
@@ -149,13 +195,13 @@ class ModelingSystem:
                             if (connection.sources is None and source is None) or (
                                 connection.sources is not None
                                 and source
-                                in [source.model_type.value for source in connection.sources]
+                                in [source.entry_type.value for source in connection.sources]
                             ):
                                 if (connection.targets is None and destination is None) or (
                                     connection.targets is not None
                                     and destination
                                     in [
-                                        target.model_type.value
+                                        target.entry_type.value
                                         for target in connection.targets
                                     ]
                                 ):
@@ -166,7 +212,7 @@ class ModelingSystem:
                 elif len(models) == 3:
                     for mediation in self.__sequence.mediations:
                         if models == [
-                            model.model_type.value
+                            model.entry_type.value
                             for model in mediation.models
                             if model is not None
                         ]:
@@ -196,36 +242,36 @@ class ModelingSystem:
             except:
                 pass
 
-        model_types = [model_type.value.upper() for model_type in ModelType]
-        remap_methods = [remap.value.lower() for remap in RemapMethod]
+        model_types = [model_type.value.upper() for model_type in EntryType]
+        remap_methods = [remap.value.lower() for remap in GridRemapMethod]
 
         if source.upper() not in model_types:
             raise KeyError(f'"{source}" not in {model_types}')
         if target is not None:
             if target.upper() not in model_types:
                 raise KeyError(f'"{target}" not in {model_types}')
-            target = ModelType(target.upper())
+            target = EntryType(target.upper())
         if method is not None:
             if method.lower() not in remap_methods:
                 raise KeyError(f'"{method}" not in {remap_methods}')
-            method = RemapMethod(method.lower())
+            method = GridRemapMethod(method.lower())
 
-        self.__sequence.connect(ModelType(source.upper()), target, method)
+        self.__sequence.connect(EntryType(source.upper()), target, method)
 
     @property
-    def connections(self) -> [str]:
+    def connections(self) -> List[str]:
         """
-        string representations of coupling connections in format `'WAV -> HYD'`
+        string representations of coupling connections in format ``'WAV -> HYD'``
         """
 
         return [str(connection) for connection in self.__sequence.connections]
 
     def mediate(
         self,
-        sources: [str] = None,
-        functions: [str] = None,
-        targets: [str] = None,
-        method: RemapMethod = None,
+        sources: List[str] = None,
+        functions: List[str] = None,
+        targets: List[str] = None,
+        method: GridRemapMethod = None,
         processors: int = None,
         **attributes,
     ):
@@ -246,8 +292,8 @@ class ModelingSystem:
             except:
                 pass
 
-        model_types = [model_type.value.upper() for model_type in ModelType]
-        remap_methods = [remap.value.lower() for remap in RemapMethod]
+        model_types = [model_type.value.upper() for model_type in EntryType]
+        remap_methods = [remap.value.lower() for remap in GridRemapMethod]
 
         if sources is not None:
             if isinstance(sources, str):
@@ -256,7 +302,7 @@ class ModelingSystem:
                 if isinstance(source, str):
                     if source.upper() not in model_types:
                         raise KeyError(f'"{source}" not in {model_types}')
-                    sources[index] = ModelType(source.upper())
+                    sources[index] = EntryType(source.upper())
         if targets is not None:
             if isinstance(targets, str):
                 targets = [targets]
@@ -264,24 +310,24 @@ class ModelingSystem:
                 if isinstance(target, str):
                     if target.upper() not in model_types:
                         raise KeyError(f'"{target}" not in {model_types}')
-                    targets[index] = ModelType(target.upper())
+                    targets[index] = EntryType(target.upper())
         if method is not None and isinstance(method, str):
             if method.lower() not in remap_methods:
                 raise KeyError(f'"{method}" not in {remap_methods}')
-            method = RemapMethod(method.lower())
+            method = GridRemapMethod(method.lower())
 
         self.__sequence.mediate(sources, functions, targets, method, processors, **attributes)
 
     @property
-    def __configuration_files(self) -> [ConfigurationFile]:
+    def __configuration_files(self) -> List[ConfigurationFile]:
         return [
             NEMSConfigurationFile(self.__sequence),
-            MeshFile(self.__sequence),
+            FileForcingsFile(self.__sequence),
             ModelConfigurationFile(self.start_time, self.duration, self.__sequence),
         ]
 
     @property
-    def configuration(self) -> {str: str}:
+    def configuration(self) -> Dict[str, str]:
         return {
             configuration_file.name: str(configuration_file)
             for configuration_file in self.__configuration_files
@@ -289,13 +335,14 @@ class ModelingSystem:
 
     def write(
         self, directory: PathLike, overwrite: bool = False, include_version: bool = False,
-    ) -> [Path]:
+    ) -> List[Path]:
         """
         write NEMS / NUOPC configuration to the given directory
 
         :param directory: path to output directory
         :param overwrite: overwrite existing files
         :param include_version: include the NEMSpy version in a comment
+        :returns: list of written file paths
         """
 
         directory = ensure_directory(directory)
@@ -310,32 +357,32 @@ class ModelingSystem:
         return filenames
 
     def __getitem__(self, model_type: str) -> ModelEntry:
-        if not isinstance(model_type, str) and not isinstance(model_type, ModelType):
+        if not isinstance(model_type, str) and not isinstance(model_type, EntryType):
             raise ValueError(
-                f'model type must be {str} or {ModelType}, not {type(model_type)}'
+                f'model type must be {str} or {EntryType}, not {type(model_type)}'
             )
-        model_types = [model_type.value.upper() for model_type in ModelType]
+        model_types = [model_type.value.upper() for model_type in EntryType]
         if model_type.upper() not in model_types:
             raise KeyError(f'"{model_type}" not in {model_types}')
-        return self.__sequence[ModelType(model_type.upper())]
+        return self.__sequence[EntryType(model_type.upper())]
 
     def __setitem__(self, model_type: str, model: ModelEntry):
-        if not isinstance(model_type, str) and not isinstance(model_type, ModelType):
+        if not isinstance(model_type, str) and not isinstance(model_type, EntryType):
             raise ValueError(
-                f'model type must be {str} or {ModelType}, not {type(model_type)}'
+                f'model type must be {str} or {EntryType}, not {type(model_type)}'
             )
-        model_types = [model_type.value.upper() for model_type in ModelType]
+        model_types = [model_type.value.upper() for model_type in EntryType]
         if model_type.upper() not in model_types:
             raise KeyError(f'"{model_type}" not in {model_types}')
-        self.__sequence[ModelType(model_type.upper())] = model
+        self.__sequence[EntryType(model_type.upper())] = model
 
     def __contains__(self, model_type: str) -> bool:
-        if not isinstance(model_type, str) and not isinstance(model_type, ModelType):
+        if not isinstance(model_type, str) and not isinstance(model_type, EntryType):
             raise ValueError(
-                f'model type must be {str} or {ModelType}, not {type(model_type)}'
+                f'model type must be {str} or {EntryType}, not {type(model_type)}'
             )
-        return ModelType(model_type.upper()) in self.__sequence
+        return EntryType(model_type.upper()) in self.__sequence
 
     def __repr__(self) -> str:
-        models = [f'{model.model_type}={repr(model)}' for model in self.__sequence.models]
+        models = [f'{model.entry_type}={repr(model)}' for model in self.__sequence.models]
         return f'{self.__class__.__name__}({repr(self.interval)}, {", ".join(models)})'
