@@ -1,22 +1,24 @@
 from os import PathLike
 
-from .base import ModelEntry, ModelMeshEntry, ModelType
+from .base import EntryType, FileForcingEntry, ModelEntry
 
 
 class AtmosphericModelEntry(ModelEntry):
     """
-    abstract implementation of a generic atmospheric model
+    abstraction of a generic atmospheric model
     """
 
-    model_type = ModelType.ATMOSPHERIC
+    entry_type = EntryType.ATMOSPHERIC
 
     def __init__(self, processors: int, **kwargs):
         super().__init__(processors, **kwargs)
 
 
-class AtmosphericMeshEntry(AtmosphericModelEntry, ModelMeshEntry):
+class AtmosphericForcingEntry(AtmosphericModelEntry, FileForcingEntry):
     """
-    Atmospheric Mesh (ATMesh) reference
+    file forcing entry for Atmospheric Mesh (ATMesh), which combines data from HRRR, GFS, etc.
+
+    >>> atmospheric_mesh = AtmosphericForcingEntry(filename='wind_atm_fin_ch_time_vec.nc', processors=1)
     """
 
     name = 'atmesh'
@@ -25,7 +27,7 @@ class AtmosphericMeshEntry(AtmosphericModelEntry, ModelMeshEntry):
         if processors is None:
             processors = 1
         AtmosphericModelEntry.__init__(self, processors, **kwargs)
-        ModelMeshEntry.__init__(self, self.model_type, filename)
+        FileForcingEntry.__init__(self, self.entry_type, filename)
 
 
 class HWRFEntry(AtmosphericModelEntry):
