@@ -1,12 +1,12 @@
-from .base import ModelEntry, ModelType
+from .base import EntryType, ModelEntry
 
 
 class OceanModelEntry(ModelEntry):
     """
-    abstract implementation of a generic oceanic model
+    abstraction of a generic oceanic model
     """
 
-    model_type = ModelType.OCEAN
+    entry_type = EntryType.OCEAN
 
     def __init__(self, processors: int, **kwargs):
         super().__init__(processors, **kwargs)
@@ -16,6 +16,8 @@ class ADCIRCEntry(OceanModelEntry):
     """
     Advanced Circulation (ADCIRC) model
     http://adcirc.org
+
+    >>> ocean_model = ADCIRCEntry(processors=11, Verbosity='max', DumpFields=False)
     """
 
     name = 'adcirc'
